If a DM device that can pass down zone append commands is stacked on top
of a device that emulates zone append commands, it will allocate zone
append emulation resources, even though it doesn't use them. This is
because the underlying device will have max_hw_zone_append_sectors set
to 0 to request zone append emulation. When the DM device is stacked on
top of it, it will inherit that max_hw_zone_append_sectors limit,
despite being able to pass down zone append bios. Solve this by making
sure max_hw_zone_append_sectors is non-zero for DM devices that do not
need zone append emulation.

Reviewed-by: Damien Le Moal <dlem...@kernel.org>
Signed-off-by: Benjamin Marzinski <bmarz...@redhat.com>
---
 drivers/md/dm-zone.c | 6 +++++-
 1 file changed, 5 insertions(+), 1 deletion(-)

diff --git a/drivers/md/dm-zone.c b/drivers/md/dm-zone.c
index 037295db1c2d..d1f8c96cee46 100644
--- a/drivers/md/dm-zone.c
+++ b/drivers/md/dm-zone.c
@@ -345,11 +345,15 @@ int dm_set_zones_restrictions(struct dm_table *t, struct 
request_queue *q,
 
        /*
         * Check if zone append is natively supported, and if not, set the
         * mapped device queue as needing zone append emulation.
         */
        WARN_ON_ONCE(queue_is_mq(q));
        if (!dm_table_supports_zone_append(t))
                lim->max_hw_zone_append_sectors = 0;
+       else if (lim->max_hw_zone_append_sectors == 0)
+               lim->max_hw_zone_append_sectors = lim->max_zone_append_sectors;
 
        /*
         * Determine the max open and max active zone limits for the mapped
@@ -345,11 +345,15 @@ int dm_set_zones_restrictions(struct dm_table *t, struct 
        /*
         * Check if zone append is natively supported, and if not, set the
-        * mapped device queue as needing zone append emulation.
+        * mapped device queue as needing zone append emulation. If zone
+        * append is natively supported, make sure that
+        * max_hw_zone_append_sectors is not set to 0.
         */
        WARN_ON_ONCE(queue_is_mq(q));
-- 
2.48.1


Reply via email to